#rsvp4gp2x - Rapid Serial Visual Projection reader for the GP2X
#Copyright (C) 2007  Kevin Granade

#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import pickle
import sys
import ConfigParser
import pygame.event
import pygame.joystick
import pygame.font
import pygame.display
import pygame.draw
import pygame.time
import pygame.mouse

from pygame.locals import *

# GP2X joystick button mappings
BUTTON_UP            = (0)
BUTTON_DOWN          = (4)
BUTTON_LEFT          = (2)
BUTTON_RIGHT         = (6)
BUTTON_UPLEFT        = (1)
BUTTON_UPRIGHT       = (7)
BUTTON_DOWNLEFT      = (3)
BUTTON_DOWNRIGHT     = (5)
BUTTON_CLICK         = (18)
BUTTON_A             = (12)
BUTTON_B             = (13)
BUTTON_X             = (14)
BUTTON_Y             = (15)
BUTTON_L             = (10)
BUTTON_R             = (11)
BUTTON_START         = (8)
BUTTON_SELECT        = (9)
BUTTON_VOLUP         = (16)
BUTTON_VOLDOWN       = (17)


#functions to be used in rsvp4gp2x.  Typically each of these will map to a buttonpress

def speedup(current_state):
  """speedup, increase speed of iteration"""
  current_state.delay -= current_state.delay>>4
  
def slowdown(current_state):
  """slowdown, reduce speed of iteration"""
  if current_state.delay < 20: current_state.delay += 1
  else: current_state.delay += current_state.delay>>4

def reverse_iteration(current_state):
  """reverse_iteration, causes iteration to proceed in reverse
  reverse_iteration and foward_iteration should be set to the buttondown and buttonup events of the same button"""
  current_state.iterate_foward = False

def foward_iteration(current_state):
  """foward_iteration, causes iteration to proceed in normal (foward) direction
  reverse_iteration and foward_iteration should be set to the buttondown and buttonup events of the same button"""
  current_state.iterate_foward = True

def clean_exit(current_state):
  """clean_exit, write bookmarks out to file and then exit"""
  if current_state.bookmarks != [0]:
    saved_bookmark_file = open(".".join([current_state.file_name_string, "bookmark"]), "w")
    pickle.dump(current_state.bookmarks, saved_bookmark_file, pickle.HIGHEST_PROTOCOL)
    saved_bookmark_file.close()
  sys.exit()

def bookmark_jump_previous(current_state):
  """bookmark_jump_previous, jump to previous bookmark
  does nothing if no bookmarks are set"""
  for mark in reversed(current_state.bookmarks):
    if mark < current_state.index:
      #only jump if bookmark is more than a trivial number of words away, otherwise
      #keep looking for an earlier bookmark
      if current_state.index - mark > 5:
        current_state.index = mark
        break

def bookmark_jump_next(current_state):
  """bookmark_jump_next, jump to next bookmark
  does nothing if no bookmarks are set"""
  for mark in current_state.bookmarks:
    if mark > current_state.index:
      current_state.index = mark
      break

def bookmark_insert(current_state):
  """bookmark_insert, add a bookmark at the current text position, might want to make it erase bookmarks in the future"""
  for num in range(len(current_state.bookmarks)):
    if current_state.index == current_state.bookmarks[num]:
      break
    if current_state.index < current_state.bookmarks[num]:
      current_state.bookmarks.insert(num, current_state.index)
      break
  else: #if the loop falls through, append to end of list
    current_state.bookmarks.insert(len(current_state.bookmarks), current_state.index)

def toggle_pause(current_state):
  """toggle_pause, this incredibly awkwardly named function toggles a variable
  which is used to pause or resume iteration of text"""
  current_state.pause = not current_state.pause

def rotate_text_counterclockwise(current_state):
  """rotate_text, rotates the text by 90 degrees each time it is invoked"""
  if current_state.rotation == 270:
    current_state.rotation = 0
  else:
    current_state.rotation += 90

def rotate_text_clockwise(current_state):
  """rotate_text, rotates the text by 90 degrees each time it is invoked"""
  if current_state.rotation == -270:
    current_state.rotation = 0
  else:
    current_state.rotation -= 90

def fast_foward_start(current_state):
  """fast_foward_start, sets the fast-foward state"""
  current_state.fast_foward = 1

def fast_foward_stop(current_state):
  """fast_foward_stop, unsets fast-foward state"""
  current_state.fast_foward = 0

def skip_foward(current_state):
  """skip_foward, moves index foward by the value of word_skip"""
  current_state.skip_words = 1

def skip_backward(current_state):
  """skip_backward, moves index foward by the value of word_skip"""
  current_state.skip_words = -1

def get_delay(current_state):
  '''get_delay, retrieves the delay set in ms
  returns string'''
  return str(current_state.delay)+" ms"

def get_wpm(current_state):
  '''get_wpm, retrieve delay converted into effective wpm
  returns string'''
  return str(60000/current_state.delay)+" WPM"

#these lists, along with the do_events_modular function provide the button-to-command mapping for the program
#I'm doing the assignment this way so I can use the labels instead of appending and having the order dictate which
#button is assigned to each function
button_down_funcs = [None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None]
button_down_funcs[BUTTON_UP] = speedup
button_down_funcs[BUTTON_UPLEFT] = None
button_down_funcs[BUTTON_LEFT] = reverse_iteration
button_down_funcs[BUTTON_DOWNLEFT] = None
button_down_funcs[BUTTON_DOWN] = slowdown
button_down_funcs[BUTTON_DOWNRIGHT] = None
button_down_funcs[BUTTON_RIGHT] = fast_foward_start
button_down_funcs[BUTTON_UPRIGHT] = None
button_down_funcs[BUTTON_START] = toggle_pause
button_down_funcs[BUTTON_SELECT] = bookmark_insert
button_down_funcs[BUTTON_L] = bookmark_jump_previous
button_down_funcs[BUTTON_R] = bookmark_jump_next
button_down_funcs[BUTTON_A] = None
button_down_funcs[BUTTON_B] = bookmark_insert
button_down_funcs[BUTTON_X] = slowdown
button_down_funcs[BUTTON_Y] = speedup
button_down_funcs[BUTTON_VOLUP] = rotate_text_clockwise
button_down_funcs[BUTTON_VOLDOWN] = rotate_text_counterclockwise
button_down_funcs[BUTTON_CLICK] = clean_exit

button_up_funcs = [None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None]
button_up_funcs[BUTTON_UP] = None
button_up_funcs[BUTTON_UPLEFT] = None
button_up_funcs[BUTTON_LEFT] = foward_iteration
button_up_funcs[BUTTON_DOWNLEFT] = None
button_up_funcs[BUTTON_DOWN] = None
button_up_funcs[BUTTON_DOWNRIGHT] = None
button_up_funcs[BUTTON_RIGHT] = fast_foward_stop
button_up_funcs[BUTTON_UPRIGHT] = None
button_up_funcs[BUTTON_START] = None
button_up_funcs[BUTTON_SELECT] = None
button_up_funcs[BUTTON_L] = None
button_up_funcs[BUTTON_R] = None
button_up_funcs[BUTTON_A] = None
button_up_funcs[BUTTON_B] = None
button_up_funcs[BUTTON_X] = None
button_up_funcs[BUTTON_Y] = None
button_up_funcs[BUTTON_VOLUP] = None
button_up_funcs[BUTTON_VOLDOWN] = None
button_up_funcs[BUTTON_CLICK] = None

def do_config(current_state):
  """do_config, reads config file and sets up configuration accordingly
  might want to set up a defaults dictionary at some point"""
  parser = ConfigParser.ConfigParser()
  parser.read("rsvp4gp2x.cfg")
  #set font values
  if parser.has_section("font"):
    for option in parser.options("font"):
      exec "current_state."+option+" = "+parser.get("font", option)

  #set misc values
  if parser.has_section("misc"):
    for option in parser.options("misc"):
      exec "current_state."+option+" = "+parser.get("misc", option)

  #assign functions to buttons
  if parser.has_section("button_mapping_down"):
    for option in parser.options("button_mapping_down"):
      exec "button_down_funcs["+option.upper()+"] = "+parser.get("button_mapping_down", option)

  if parser.has_section("button_mapping_up"):
    for option in parser.options("button_mapping_up"):
      exec "button_up_funcs["+option.upper()+"] = "+parser.get("button_mapping_up", option)

def usage():
  print "usage: rsvp.py -t text_file | -z zipped_text_file | -h"

def do_events_modular(current_state):
  '''do_events_modular, provides a framework for arbitrarily remapping controls
  Gp2x buttons are mapped from 0 to 18, so we have a list of 19 functions we can map.
  This function will call into that list.  It's messy using the raw values, so we'll se how it goes

  returns None'''
  event_list = pygame.event.get()
  for event in event_list:
    if event.type == pygame.QUIT: clean_exit()
    elif event.type == pygame.JOYBUTTONDOWN:
      if button_down_funcs[event.button] != None:
        button_down_funcs[event.button](current_state)
    elif event.type == pygame.JOYBUTTONUP:
      if button_up_funcs[event.button] != None:
        button_up_funcs[event.button](current_state)

def do_events_keyboard(current_state):
  pygame.event.pump()
  keycodes = pygame.key.get_pressed()
  if keycodes[K_q]: clean_exit(current_state)
  elif keycodes[K_UP]: speedup(current_state)
  elif keycodes[K_LEFT]: skip_backward(current_state)
  elif keycodes[K_RIGHT]: skip_foward(current_state)
  elif keycodes[K_PERIOD]: bookmark_jump_next(current_state)
  elif keycodes[K_COMMA]: bookmark_jump_previous(current_state)
  elif keycodes[K_b]: bookmark_insert(current_state)
  elif keycodes[K_DOWN]: slowdown(current_state)
  elif keycodes[K_p]: toggle_pause(current_state)
